unit Uimgform;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Menus;

type
  TImgForm = class(TForm)
    img: TImage;
    MainMenu1: TMainMenu;
    mFile: TMenuItem;
    mFileGrabScreenshot: TMenuItem;
    mFileBar1: TMenuItem;
    mFileExit: TMenuItem;
    mFileSave: TMenuItem;
    mFileBar2: TMenuItem;
    sdSave: TSaveDialog;
    mView: TMenuItem;
    mViewZoomIn: TMenuItem;
    mViewZoomOut: TMenuItem;
    mViewResetZoom: TMenuItem;
    mView200Zoom: TMenuItem;
    mView400Zoom: TMenuItem;
    mEdit: TMenuItem;
    mEditCrop: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure mFileSaveClick(Sender: TObject);
    procedure mFileGrabScreenshotClick(Sender: TObject);
    procedure mFileExitClick(Sender: TObject);
    procedure mViewZoomInClick(Sender: TObject);
    procedure mViewZoomOutClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure mViewResetZoomClick(Sender: TObject);
    procedure mView200ZoomClick(Sender: TObject);
    procedure mView400ZoomClick(Sender: TObject);
    procedure imgMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure mEditCropClick(Sender: TObject);
  private
    FZoom: Integer;
    procedure SetZoom(AZoom: Integer);
  public
    property Zoom: Integer read FZoom write SetZoom;
  end;

var
  ImgForm: TImgForm;

implementation

uses
  MainUnit, UCropBox;

{$R *.DFM}

procedure TImgForm.SetZoom(AZoom: Integer);
begin
  if (AZoom < 1) or (AZoom > 64) then Exit;
  if AZoom=FZoom then Exit;
  FZoom:=AZoom;
  img.Width:=(LongInt(img.Picture.Bitmap.Width)*AZoom) div 16;
  img.Height:=(LongInt(img.Picture.Bitmap.Height)*AZoom) div 16;
end;

procedure TImgForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Main.lbPos.Caption:='';
  Action:=caFree;
end;

procedure TImgForm.mFileSaveClick(Sender: TObject);
begin
  if sdSave.Execute then try
    img.Picture.SaveToFile(sdSave.FileName);
  except
    ShowMessage('Failed to save the file');
  end;
end;

procedure TImgForm.mFileGrabScreenshotClick(Sender: TObject);
begin
  Main.mFileGrabScreenshot.Click;
end;

procedure TImgForm.mFileExitClick(Sender: TObject);
begin
  Main.mFileExit.Click;
end;

procedure TImgForm.mViewZoomInClick(Sender: TObject);
begin
  Zoom:=Zoom + 1;
end;

procedure TImgForm.mViewZoomOutClick(Sender: TObject);
begin
  Zoom:=Zoom - 1;
end;

procedure TImgForm.FormCreate(Sender: TObject);
begin
  FZoom:=16;
end;

procedure TImgForm.mViewResetZoomClick(Sender: TObject);
begin
  Zoom:=16;
end;

procedure TImgForm.mView200ZoomClick(Sender: TObject);
begin
  Zoom:=32;
end;

procedure TImgForm.mView400ZoomClick(Sender: TObject);
begin
  Zoom:=64;
end;

procedure TImgForm.imgMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  Main.UpdatePos(LongInt(X)*16 div Zoom, LongInt(Y)*16 div Zoom);
end;

procedure TImgForm.mEditCropClick(Sender: TObject);
var
  X, Y, X1, Y1, X2, Y2: Integer;
  NewBmp: TBitmap;
begin
  Application.CreateForm(TCropBox, CropBox);
  CropBox.seLeft.MaxValue:=img.Picture.Bitmap.Width - 1;
  CropBox.seRight.MaxValue:=img.Picture.Bitmap.Width - 1;
  CropBox.seTop.MaxValue:=img.Picture.Bitmap.Height - 1;
  CropBox.seBottom.MaxValue:=img.Picture.Bitmap.Height - 1;
  CropBox.seRight.Value:=CropBox.seRight.MaxValue;
  CropBox.seBottom.Value:=CropBox.seBottom.MaxValue;
  if CropBox.ShowModal=mrOk then begin
    X1:=CropBox.seLeft.Value;
    Y1:=CropBox.seTop.Value;
    X2:=CropBox.seRight.Value;
    Y2:=CropBox.seBottom.Value;
    if not ((X1=0) and (Y1=0) and (X2=img.Picture.Bitmap.Width - 1) and (Y2=img.Picture.Bitmap.Height - 1)) then begin
      NewBmp:=TBitmap.Create;
      NewBmp.Width:=X2 - X1 + 1;
      NewBmp.Height:=Y2 - Y1 + 1;
      for Y:=Y1 to Y2 do
        for X:=X1 to X2 do
          NewBmp.Canvas.Pixels[X - X1, Y - Y1]:=img.Picture.Bitmap.Canvas.Pixels[X, Y];
      if CropBox.cbCreateNew.Checked then begin
        Application.CreateForm(TImgForm, ImgForm);
        ImgForm.Caption:=Caption + ' Copy';
        ImgForm.img.Width:=NewBmp.Width;
        ImgForm.img.Height:=NewBmp.Height;
        ImgForm.img.Picture.Assign(NewBmp);
        Main.lbStatus.Caption:='Created new ' + IntToStr(NewBmp.Width) + 'x' + IntToStr(NewBmp.Height) + ' image';
      end else begin
        img.Picture.Assign(newBmp);
        Zoom:=16;
        Main.lbStatus.Caption:='Cropped the image to ' + IntToStr(NewBmp.Width) + 'x' + IntToStr(NewBmp.Height);
      end;
      NewBmp.Free;
    end;
  end;
  CropBox.Free;
end;

end.
